﻿class HUD extends MovieClip
{
	//Integers 
	private var int_Score:Number;
	private var int_Fuel:Number;
	private var int_Lives:Number;
	//Float
	private var flo_CountingScore:Number = 0;
	//Constants 
	private var INT_MAXFUEL:Number = 500;
	//Arrays
	private var ary_LevelScores:Array;
	//Booleans
	private var bool_StartScoreEffect:Boolean = false;
	private var bool_StartFuelEffect:Boolean = false;
	private var bool_GameOver:Boolean = false;
	private var bool_FadeIn:Boolean = true;
	
	public function onLoad()
	{
		//make sure the player has fuel when the game loads
		refuelPlayer();
		//make sure the players has lives when the game loads
		refillLives();
		//make sure the score is reset
		int_Score = 0;
		//make this invisible so it can be faded in
		this._alpha = 0;
	}
	
	public function onEnterFrame()
	{
		//special effect for the end of level score text
		if(bool_StartScoreEffect)
			runTextScoreEffect();
			
		//special effect for the refilling of the fuel bar
		if(bool_StartFuelEffect)
			runFuelEffect();
		else
			//re draw the fuel bar to show the players current fuel
			_root[this._name].mov_FuelBar._xscale = (int_Fuel/INT_MAXFUEL) * 100;
		
		//Special effect for the top score when it get's updated 
		if(flo_CountingScore <= int_Score)
		{
			//update the counting score until it's the same as the score
			flo_CountingScore += (int_Score - flo_CountingScore) * 0.1;
			//update the onscreen text 
			drawScore();
		}
		
		//we use this to fade in and out the HUD at the start and the end of the game
		if(bool_FadeIn && this._alpha < 100)
			this._alpha += 5;
		else if(!bool_FadeIn && this._alpha > 0)
			this._alpha -= 5;
	}
	
	public function takeFuel():Boolean
	{
		//check if the player has fuel 
		if(int_Fuel > 0)
		{
			//remove fuel because the player is about to thrust
			int_Fuel--;
			//allow the player to move
			return true;
		}
		else
			//don't allow the player to move becaus they have no fuel
			return false;
	}
	
	public function takeLife()
	{
		//check if the player has any lives
		if(int_Lives > 1)
		{
			//remove the current life images
			_root[this._name]["mov_Life" + int_Lives]._visible = false;
			//Take away a life
			int_Lives--;
		}
		else
		{
			//the last life doesn't remove so we have to clear it now 
			_root[this._name]["mov_Life1"]._visible = false;
			//gameover if the player has no lives
			bool_GameOver = true;
		}
	}
	
	public function updateScore(sentScore:Number):Void
	{
		//update the score with the sent value
		int_Score += sentScore;
	}
	
	public function refuelPlayer():Void
	{
		//run the refuel effect
		bool_StartFuelEffect = true;
		//make sure the actual fuel is full
		int_Fuel = INT_MAXFUEL;
	}
	
	public function attachLevelText():Void
	{
		//attach the level text to tell the user the level they are currently on
		_root.attachMovie("LevelText","LevelText",_root.getNextHighestDepth(),{_x:215,_y:68});
		//make sure the text says the right level
		_root["LevelText"].txt_Level.text = "Level " + _root.int_CurrentLevel;
	}
	
	public function attachGameOver():Void
	{
		//attach the text and button showing the player has finish the game
		_root.attachMovie("GameOver","GameOver",_root.getNextHighestDepth(),{_x:300,_y:115});
		//load in the score for the onscreen text
		_root["GameOver"].int_FinalScore = int_Score;
		//set up a blank variable to add to to get a counting effect
		_root["GameOver"].flo_CounterToFinalScore = 0;
		//make the movieclip invisible so it can be faded in
		_root["GameOver"]._alpha = 0;
		//now set up the counting and fading in of the movieclip
		//A whole class isn't need for this movieclip so we can set it all up here
		_root["GameOver"].onEnterFrame = function()
		{
			//fade in the game over movieclip
			if(this._alpha < 100)
				this._alpha += 5;
				
			//add to the score for effect
			if(this.flo_CounterToFinalScore < Math.round(this.int_FinalScore))
				this.flo_CounterToFinalScore += (this.int_FinalScore - this.flo_CounterToFinalScore) * 0.1;
			
			//set up the gameover text
			_root["GameOver"].txt_FinalScore.text = Math.round(this.flo_CounterToFinalScore);
			//start a loop for adding extra zeros
			for(var i:Number = 0; i < 9 -(_root["GameOver"].txt_FinalScore.text).length; i++)
			{
				_root["GameOver"].txt_FinalScore.text = "0" + _root["GameOver"].txt_FinalScore.text;
			}
		}
		
		//next we need to set up a function so ge button will take us to the main menu
		_root["GameOver"].btn_ReturnHome.onPress = function()
		{
			//all movieclip other than the player, HUD and the Level manager should be removed so
			//we need to remove these movieclips and attach the main menu
			_root["Player"].removeMovieClip();
			_root.lm_LevelManager = null;
			//now load the main menu to the center of the screen
			_root.attachMovie("MainMenu","MainMenu",_root.getNextHighestDepth(),{_x: 300, _y:200});
			//and finially remove the GameOver Text 
			_root["GameOver"].removeMovieClip();
			//we actually need to remove the HUD last so all the above function don't get cut off
			_root["HUD"].removeMovieClip();
		}
	}
	
	public function attachLevelScore(sentLandingScore:Number):Void
	{
		//attach the level score so it can be filled in
		_root.attachMovie("LevelScore","LevelScore",_root.getNextHighestDepth(),{_x:200,_y:68});
		
		//if the player has no fuel they get no score which is a bit harsh so we check if
		//the fuel if less than or equal to 0
		if(int_Fuel <= 0)
			//and set it to one 
			int_Fuel = 1;
		//store all the current level scores in an array
		ary_LevelScores = [[sentLandingScore, 0], [int_Fuel, 0], [sentLandingScore * int_Fuel, 0]];
		//Flag up that the effect for the score should start
		bool_StartScoreEffect = true;
	}
	
	public function isGameOver():Boolean
	{
		 return bool_GameOver;
	}
	
	public function fadeOut():Void
	{
		bool_FadeIn = false;
	}
	
	private function drawScore():Void
	{
		//creat a temp variable to store the formatted score
		//initalise it with the current score in string form
		var tempFormattedScore:String = String(Math.round(flo_CountingScore));
		//Add the necessary number of zero to keep the string at a standard length
		for(var i:Number = 0; i < 8 - String(Math.round(flo_CountingScore)).length; i++)
		{
			tempFormattedScore = "0" + tempFormattedScore;
		}
			
		//print out the formatted string
		_root[this._name].txt_Score.text = tempFormattedScore;
	}
	
	private function refillLives():Void
	{
		//Set the lives to 3
		int_Lives = 3;
		//make all the lives visible again
		for(var i:Number = 1; i <= int_Lives; i++)
		{
			_root[this._name]["mov_Life" + i]._visible = true;
		}
	}
	
	private function runTextScoreEffect():Void
	{ 
		//create a temp variable to hold the current text that needs incremented
		var tempCurrentTextNumber:Number;
			
		//check which bit of text needs to be incremented
		if(ary_LevelScores[0][0] > Math.round(ary_LevelScores[0][1]))
			tempCurrentTextNumber = 0;
		else if(ary_LevelScores[1][0] > Math.round(ary_LevelScores[1][1]))
			tempCurrentTextNumber = 1;
		else if(ary_LevelScores[2][0] > Math.round(ary_LevelScores[2][1]))
			tempCurrentTextNumber = 2;
		else
		{
			//update the players score because we're done
			updateScore(ary_LevelScores[2][0]);
			//stop the loop from happening
			bool_StartScoreEffect = false;
			//move onto the next level
			_root.lm_LevelManager.finishLevel();
			//leave the method because we don't need to change anything
			return;
		}
		//add 10% of the current gap to the number
		ary_LevelScores[tempCurrentTextNumber][1] += (ary_LevelScores[tempCurrentTextNumber][0] - ary_LevelScores[tempCurrentTextNumber][1]) * 0.1;
		   
		//print out all scores because it's quicker than using an switch
		//statment to work it out(He says. it probably uses more resources)
		//I'm just too lazy to write the statement. Shouldn't really matter :P)
		_root["LevelScore"].txt_LandingPoints.text = Math.round(ary_LevelScores[0][1]);
		_root["LevelScore"].txt_FuelPoints.text = Math.round(ary_LevelScores[1][1]);
		_root["LevelScore"].txt_TotalScore.text = Math.round(ary_LevelScores[2][1]);
	}
	
	private function runFuelEffect():Void
	{
		//instead of the fuel bar magically fully refilling we make it slowly increase
		if(_root[this._name].mov_FuelBar._xscale < 100)
			_root[this._name].mov_FuelBar._xscale++;
		else
			bool_StartFuelEffect = false;
	}
}