﻿//Import the glow filter for use when the player successfully lands 
import flash.filters.GlowFilter;

class Player extends MovieClip
{
	//Constants
	private var INT_ROTATEANGLE:Number = 3;	
	private var FLO_GRAVITY:Number = 0.010;
	private var FLO_SPEED:Number = FLO_GRAVITY * 3;
	private var FLO_FRICTION:Number = 1;
	private var FLO_MAXVELOCITY:Number = 5.0;
	private var FLO_COLLISIONSRADIUS:Number = 10;
	private var FLO_MAXLANDINGSPEED:Number = 0.5;
	private var INT_LEGHEIGHT:Number = 3;
	//Floats
	private var flo_XVelocity:Number;
	private var flo_YVelocity:Number;
	private var flo_AngleInDegrees:Number;
	//Boolean
	private var bool_GamePaused:Boolean;
	private var bool_FadePlayer:Boolean;
	//Integers
	private var int_ResetPlayerTimer:Number;
	
	public function onLoad()
	{
		//initalise the player velocity
		resetVelocity();
		//make sure the thurster doesn't show
		_root[this._name].Thrust._yscale = 0;
		//pause the game whilst everything fades in
		bool_GamePaused = true;
		//make this invisible so we can fade it in 
		this._alpha = 0;
		
	}
	
	public function onEnterFrame()
	{
		//check if the game has been pause before doing anthing
		if(!bool_GamePaused)
		{
			//Call function that handles everything to move the player
			movePlayer();
			//Call function to detect all collisions with walls and other objects
			checkCollisions();
		}
		
		//When the player is dead we need to fade them out
		if(bool_FadePlayer)
			if(this._alpha > 0)
				this._alpha -= 2;
			else
				//when the player is faded out we can stop fading them
				bool_FadePlayer = false;
	}	
	
	public function setPosition(sentX:Number, sentY:Number):Void
	{
		//set the x and y values to the sent parameters
		setX(sentX);
		setY(sentY);
	}
	
	public function	setX(sentX:Number):Void
	{
		this._x = sentX;
	}
	
	public function setY(sentY:Number):Void
	{
		this._y = sentY;
	}
	
	public function resetVelocity():Void
	{
		//set both the x and y velocity to 0
		flo_XVelocity = 0;
		flo_YVelocity = 0;
	}
	
	public function unpauseGame():Void
	{
		bool_GamePaused = false;
	}
	
	public function resetPlayer():Void
	{
		//clear the interval used to call this funciton 
		clearInterval(int_ResetPlayerTimer);
		//clear the player velocity
		resetVelocity();
		//refuel the player
		_root["HUD"].refuelPlayer();
		//put the player back to it's original starting position
		setPosition(100, 58);
		//make the player visible again
		this._alpha = 100;
		//reset the player's rotation
		this._rotation = 0;
		//unpause the game
		unpauseGame();
	}
	
	private function movePlayer():Void
	{
		//Controls for the game are both the Arrows and WASD for movement
		// and spacebar for thrust 
		
		//Check if the player is trying to rotate left(Left Arrow OR A)
		if(Key.isDown(Key.LEFT) || Key.isDown(65))
		{
			this._rotation -= INT_ROTATEANGLE;
		}
		//Check if the player is trying to rotate right(Right Arrow OR D)
		else if(Key.isDown(Key.RIGHT) || Key.isDown(68))
		{
			//rotate the player right by the pre-set angle
			this._rotation += INT_ROTATEANGLE;
		}
		//Check if the player is trying to thurst(Spacebar, Up Arrow or W)
		//and if they have enough fuel for the thruster to take away
		if((Key.isDown(Key.SPACE) || Key.isDown(87) || Key.isDown(Key.UP)) && _root["HUD"].takeFuel())
		{
			//work out the players current angle in radians
			flo_AngleInDegrees = (this._rotation/360) * 2 * Math.PI;
			//update the player x&y velocity based on it's current angle 
			flo_XVelocity += FLO_SPEED * Math.sin(flo_AngleInDegrees);
			flo_YVelocity -= FLO_SPEED * Math.cos(flo_AngleInDegrees);
			//adjust the thruster graphic to make it look like its moving
			if(_root[this._name].Thrust._yscale <= 90)
				_root[this._name].Thrust._yscale += 10;
				
		}
		else ////------------ May need to redo this thruster image shit ------------////////	
			//reset the thrust image
			_root[this._name].Thrust._yscale = 0;
		
		//apply gravity
		flo_YVelocity += FLO_GRAVITY;
		
		//check that the Player isn't exceeding the max velocity 
		//if they are reset it 
		if(flo_XVelocity > FLO_MAXVELOCITY)
			flo_XVelocity = FLO_MAXVELOCITY;
		else if(flo_XVelocity < -FLO_MAXVELOCITY)
			flo_XVelocity = -FLO_MAXVELOCITY;
			
		if(flo_YVelocity > FLO_MAXVELOCITY)
			flo_YVelocity = FLO_MAXVELOCITY;
		else if(flo_YVelocity < -FLO_MAXVELOCITY)
			flo_YVelocity = -FLO_MAXVELOCITY;
			
		//update the players x and y positions and apply a little friction
		this._x += flo_XVelocity * FLO_FRICTION;
		this._y += flo_YVelocity * FLO_FRICTION;
		
	}
	
	private function checkCollisions():Void
	{
		//When the player goes off either the left or right side they should reappear on the otherside
		//this function handles that as well as if the player tries to go off the top and Bottom of the screen
		screenCollision();
		
		//Due to the way the ground collision is done the player can be just about to land on a zone but 
		//the game could possibly kill them so in the landing zone code we correct this by making the player
		//invincible when they're above wholey above the landing zone. This problem could be solved by 
		//removing the side check on the lines but the other way just covers all bases I believe
		if(!landingZoneCollision())
			//We need to check if the player is colliding with the ground
			groundCollision();		
	}
	
	private function screenCollision():Void
	{
		//first we check if the player has gone off the left side of the screen
		//we also acount for the point of reference being the middle of the movieclip
		//and adjusting the value so the whole ship is off screen before repositioning it
		if(this._x < 0 - (this._width/2))
			//again adjust the ship position when attaching it off screen
			setX(Stage.width + (this._width/2));
		//We do the same again but for the right side
		else if(this._x > Stage.width + (this._width/2))
			//again adjust the ship position when attaching it off screen
			setX(0 - (this._width/2));
			
		//next we want to stop the player going off the top and Bottom of the screen
		
		//if the player hits the top of the screen they should lose momentum
		if(this._y < 0 + (this._height/2))
		{
			//to stop the player getting stuck we need to set the y > what required
			//by the if statement
			this._y = (this._height/2);
			//make the ship bounce back but with only 10% of it's current y velocity
			flo_YVelocity = -(flo_YVelocity/10)
		}
		// now if the player is about to hit the Bottom of the screen we do pretty much the same
		else if(this._y > Stage.height - (this._height/2))
		{
			this._y = Stage.height - (this._height/2);
			flo_YVelocity = -(flo_YVelocity/10);
		}
	}
	
	private function groundCollision():Void
	{
		//Set up holding for the length of the line
		var tempLineLength:Number = 0;
		//Set up holding for the gap between the center of the player and the
		//top and Bottom of the line
		var tempDistanceFromTop:Number = 0;
		var tempDistanceFromBottom:Number = 0;
		//Storage for the angles used to work out the distance of the player for the line
		var tempTopAngle:Number = 0;
		var tempBottomAngle:Number = 0;
		//Storage for the distance between the player and the line
		var tempDistanceFromLine:Number = 0;
		
		//Start loop to go through all ground lines currently on screen 
		for(var i:Number = 0; i < _root.lm_LevelManager.getCurrentNumberOfLines(); i++)
		{
			//get the current ground 
			var tempGround:MovieClip = _root["Land" + i];
		
			//First we need to get the length of the line
			tempLineLength = Math.sqrt(Math.pow(tempGround.ary_Positions[1][0] - tempGround.ary_Positions[0][0], 2) + Math.pow(tempGround.ary_Positions[1][1] - tempGround.ary_Positions[0][1], 2));
		
			//Work out the distance between the top of the line and the center of the player
			tempDistanceFromTop = Math.sqrt(Math.pow(this._y - tempGround.ary_Positions[0][1], 2) + Math.pow(this._x - tempGround.ary_Positions[0][0],2));
			//To help towards proformance we'll check if this distance from the top of the line
			//is less than the hit radius of the player
			if(tempDistanceFromTop <= FLO_COLLISIONSRADIUS)
			{
				playerCrashed();
				//Break from the loop so we don't keep checking other lines
				break;
			}
			else
			{
				//Work out the distance between the Bottom of the line and the center of the player
				tempDistanceFromBottom = Math.sqrt(Math.pow(this._y - tempGround.ary_Positions[1][1], 2) + Math.pow(this._x - tempGround.ary_Positions[1][0], 2));
				// Again we check if the distance is less than the hit radius but for the Bottom of the line
				if(tempDistanceFromBottom <= FLO_COLLISIONSRADIUS)
				{
					playerCrashed();
					//Break from the loop so we don't keep checking other lines
					break;
				}
				else
				{
					//work out the angle of between the tempDistanceFromTop and tempLineLength
					tempTopAngle = Math.acos((Math.pow(tempDistanceFromTop, 2) + Math.pow(tempLineLength, 2) - Math.pow(tempDistanceFromBottom, 2))/(2 * tempDistanceFromTop * tempLineLength));
					//this is only used for a check 
					tempBottomAngle = Math.acos((Math.pow(tempDistanceFromBottom, 2) + Math.pow(tempLineLength, 2) - Math.pow(tempDistanceFromTop, 2))/(2 * tempDistanceFromBottom * tempLineLength));
					//Now work out the distance between the line and the player
					tempDistanceFromLine = tempDistanceFromTop * Math.sin(tempTopAngle);
					//finially check if the two movieclips have collided based on the distance from line
					//and check that both angles are less than 90 degrees so a right angles triangle can be created
					if(tempDistanceFromLine <= FLO_COLLISIONSRADIUS && tempTopAngle < Math.PI/2 && tempBottomAngle < Math.PI/2)
					{
						playerCrashed();
						//Break from the loop so we don't keep checking other lines
						break;
					}
				}
			}
		}
	}
	
	private function landingZoneCollision():Boolean
	{
		//Loop through all 4 landing zones
		for(var i:Number = 0; i < 4; i++)
		{
			//first check if the player is closely above the current landing zone
			if((this._y - FLO_COLLISIONSRADIUS) > (_root["LandingZone" + i].ary_Positions[0][1] - (4 * FLO_COLLISIONSRADIUS)) && (this._y + FLO_COLLISIONSRADIUS) < _root["LandingZone" + i].ary_Positions[0][1] + _root["LandingZone" + i].int_Height
			   	&& (this._x + FLO_COLLISIONSRADIUS)< _root["LandingZone" + i].ary_Positions[1][0] && (this._x - FLO_COLLISIONSRADIUS) > _root["LandingZone" + i].ary_Positions[0][0])
			{
				//Now we know the player is in the range of the zone we can check if they have hit it
				//we also include the leg height as well so the player gets a more realisict landing
				if(this._y >= _root["LandingZone" + i].ary_Positions[0][1] - (FLO_COLLISIONSRADIUS+ INT_LEGHEIGHT))
				{
					//if the current speed of the player is slow enough annd that the player is the 
					//right way up we can show that the player has landed
					if(this._rotation <= INT_ROTATEANGLE && this._rotation >= -INT_ROTATEANGLE
					   && flo_XVelocity > -FLO_MAXLANDINGSPEED && flo_XVelocity < FLO_MAXLANDINGSPEED && flo_YVelocity > -FLO_MAXLANDINGSPEED && flo_YVelocity < FLO_MAXLANDINGSPEED)
						//show that the level has been beat
						levelWon(_root["LandingZone" + i]);
					//if not we show that they have crashed into the landing zone
					else
						playerCrashed();
				}
				//return true so the other collision testing doesn't run
				return true;
			}
		}
		return false;
	}
	
	private function levelWon(sentLandingZone:MovieClip):Void
	{
		//First pause the game so the player can't move
		bool_GamePaused = true;
		//rotate the player so they're straight
		this._rotation = 0;
		//clear the players velocity
		resetVelocity();
		//make sure the player and legs are touching the zone
		this._y = sentLandingZone.ary_Positions[0][1] - (FLO_COLLISIONSRADIUS + INT_LEGHEIGHT);
		//make the thrust off as well because it can glitch on
		_root[this._name].Thrust._yscale = 0;
		//attach the glow filter to the zone the player has just landed on
		sentLandingZone.filters = new Array(new GlowFilter(0xFFFFFF, .9, 5, 5, 2, 3, false, false));
		//attach the score text so the player knows how much they have scored on the current level
		_root["HUD"].attachLevelScore(sentLandingZone.int_RewardScore);
	}
	
	private function playerCrashed():Void
	{
		//When the player crashes we need to show an explosion
		explodePlayer();
		//then we need to stop the player and fade it out
		bool_GamePaused = true;
		bool_FadePlayer = true;
		//take away a life
		_root["HUD"].takeLife()
		//check if the game is not over
		if(!_root["HUD"].isGameOver())
			//set a timer that calls a function to set the player back to the start
			int_ResetPlayerTimer = setInterval(this,"resetPlayer",2000);
		else
			_root.lm_LevelManager.finishLevel();
	}
	
	private function explodePlayer():Void
	{
		//A variable to store the current reference movieclip
		var tempMovieClipStore:MovieClip;
		//for the explosion we attach particals and make the push away 
		//and fade out from the center of the ship
		for(var i:Number = 0; i < 10; i++)
		{
			tempMovieClipStore = _root.attachMovie("ExplosionPartical", "ExplosionPartical" + i,_root.getNextHighestDepth(),{_x:this._x, _y:this._y});
			//we rotate the mpvie clip to the dirction we want to move it 
			tempMovieClipStore._rotation = (i * 36);
			//now create the onEnterFrame function to handle the player fading and movement
			tempMovieClipStore.onEnterFrame = function()
			{
				//if the partical is invisible remove them
				if(this._alpha < 0)
					this.removeMovieClip();
				else
					//else fade the partical out
					this._alpha -= 2;
				//whilst the partical is fading move it away from it's current the position
				//using it current rotation as the angle
				this._x += Math.cos(this._rotation * (Math.PI / 180)) * 0.6;
				this._y += Math.sin(this._rotation * (Math.PI / 180)) * 0.6;
			}
		}
	}
}